<?xml version="1.0"?>
<!DOCTYPE gui SYSTEM "kpartgui.dtd">
<gui version="423" name="kontact" >
<MenuBar>
  <Menu name="file" noMerge="1">
    <text>&amp;File</text>
    <Merge/>
    <Separator/>
    <Action name="file_quit"/>
    <Merge/>
  </Menu>
  <Merge />
  <Menu noMerge="1" name="view">
     <text>&amp;View</text>
     <Action name="hide_show_sidebar"/>
     <Separator/>
     <Action name="fullscreen"/>
  </Menu>
  <Menu noMerge="1" name="settings">
     <text>&amp;Settings</text>
     <Merge name="StandardToolBarMenuHandler" />
     <Action name="options_show_menubar" group="settings_configure" />
     <Merge/>
     <Separator/>
     <Action name="colorscheme_menu" />
     <Separator/>
     <Action name="switch_application_language" group="settings_configure"/>
     <Action name="options_configure_keybinding"/>
     <Action name="options_configure_toolbars"/>
     <DefineGroup name="settings_configure"/>
     <Action name="settings_configure_kontact"/>
  </Menu>
  <Menu name="help"><text>&amp;Help</text>
     <Action name="help_introduction" append="help_whats_this_merge"/>
   <Separator/>
   <Action name="verify_check_version" />
   <Separator/>
  </Menu>
</MenuBar>
<ToolBar position="Top" noMerge="1" name="mainToolBar"><text>Main Toolbar</text>
  <Action name="action_new"/>
  <Action name="action_sync"/>
  <Merge/>
</ToolBar>
<Merge/>
<ToolBar position="Top" noMerge="1" hidden="true" name="navigatorToolBar"><text>Navigator</text>
  <ActionList name="navigator_actionlist" />
</ToolBar>
</gui>
